from setuptools import setup, find_packages

setup(
    name='ssm_pca',
    version='0.1.0',
    author='Leonor Lopes',
    author_email='leonorserranolopes@gmail.com',
    packages=find_packages(),
    install_requires=[
        # TODO: add dependencies
    ],
    licence='MIT'
)